@@ -10,6 +10,10 @@
 #ifndef NCCONFIGURE_H
 #define NCCONFIGURE_H 1
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -41,37 +45,78 @@ typedef SSIZE_T ssize_t;
 #ifndef _WIN32
 #if __STDC__ == 1 /*supposed to be same as -ansi flag */
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 /* WARNING: in some systems, these functions may be defined as macros, so check */
+#ifndef HAVE_STRDUP
 #ifndef strdup
-extern char* strdup(const char*);
+char* strdup(const char*);
 #endif
+#endif
+
+#ifndef HAVE_STRLCAT
 #ifndef strlcat
-extern size_t strlcat(char*,const char*,size_t);
+size_t strlcat(char*,const char*,size_t);
 #endif
+#endif
+
+#ifndef HAVE_SNPRINTF
 #ifndef snprintf
-extern int snprintf(char*, size_t, const char*, ...);
+int snprintf(char*, size_t, const char*, ...);
 #endif
+#endif
+
+#ifndef HAVE_STRCASECMP
 #ifndef strcasecmp
 extern int strcasecmp(const char*, const char*);
 #endif
+#endif
+
+#ifndef HAVE_STRTOLL
 #ifndef strtoll
-extern long long int strtoll(const char*, char**, int);
+long long int strtoll(const char*, char**, int);
 #endif
+#endif
+
+#ifndef HAVE_STRTOULL
 #ifndef strtoull
-extern unsigned long long int strtoull(const char*, char**, int);
+unsigned long long int strtoull(const char*, char**, int);
 #endif
-#ifndef fileno
-extern int fileno(FILE*);
+#endif
+
+#if defined(__cplusplus)
+}
 #endif
 
 #endif /*STDC*/
-#endif /*!_WIN32*/
 
-#ifdef _WIN32
+#else /*_WIN32*/
+
 #ifndef HAVE_STRLCAT
 #define strlcat(d,s,n) strcat_s((d),(n),(s))
 #endif
+
+
+#ifndef __MINGW32__
+#ifndef strcasecmp
+#define strcasecmp _stricmp
 #endif
+#ifndef strncasecmp
+#define strncasecmp _strnicmp
+#endif
+#ifndef snprintf
+#if _MSC_VER<1900
+#define snprintf _snprintf
+#endif
+#endif
+#ifndef fileno
+#define fileno(f) _fileno(f)
+#endif
+#endif /*__MINGW32__*/
+
+#endif /*_WIN32*/
 
 /* handle null arguments */
 #ifndef nulldup
@@ -129,6 +174,10 @@ typedef unsigned long long uint64_t;
 typedef unsigned long long size64_t;
 #endif
 
+#ifndef HAVE_PTRDIFF_T
+typedef long ptrdiff_t;
+#endif
+
 /* Provide a fixed size alternative to off_t or off64_t */
 typedef long long fileoffset_t;
 
@@ -136,5 +185,4 @@ typedef long long fileoffset_t;
 #define NC_UNUSED(var) (void)var
 #endif
 
-
 #endif /* NCCONFIGURE_H */
