@@ -15,14 +15,17 @@
 
 #define STSIZE 1000
 
-#if !defined _WIN32 && !defined __CYGWIN__
+#ifdef H5BACKTRACE
+#  if !defined _WIN32 && !defined __CYGWIN__
 static void* stacktrace[STSIZE];
+#  endif
 #endif
 
 int
 nch5breakpoint(int err)
 {
-#if !defined _WIN32 && !defined __CYGWIN__
+#ifdef H5BACKTRACE
+#  if !defined _WIN32 && !defined __CYGWIN__
     int count = 0;
     char** trace = NULL;
     int i;
@@ -32,17 +35,19 @@ nch5breakpoint(int err)
     fprintf(stderr,"backtrace:\n");
     for(i=0;i<count;i++)
         fprintf(stderr,"[%03d] %s\n",i,trace[i]);
-#if 0
+#    if 0
     if(trace != NULL) free(trace);
-#endif
+#    endif
+#  endif
 #endif
     return err;
 }
 
 int
-nch5throw(int err)
+nch5throw(int err, int line)
 {
     if(err == 0) return err;
+    fprintf(stderr,">>> hdf5throw: line=%d err=%d\n",line,err); fflush(stderr);
     return nch5breakpoint(err);
 }
 #endif
