@@ -19,13 +19,12 @@ static unsigned int optimize = 0;
 extern int NCZ_buildchunkkey(size_t R, const size64_t* chunkindices, char** keyp);
 
 /* 0 => no debug */
-static unsigned int wdebug = 0;
+static unsigned int wdebug = 1;
 
 /* Forward */
 static int NCZ_walk(NCZProjection** projv, NCZOdometer* chunkodom, NCZOdometer* slpodom, NCZOdometer* memodom, const struct Common* common, void* chunkdata);
 static int rangecount(NCZChunkRange range);
 static int readfromcache(void* source, size64_t* chunkindices, void** chunkdata);
-static int NCZ_fillchunk(void* chunkdata, struct Common* common);
 static int iswholechunk(struct Common* common,NCZSlice*);
 static int wholechunk_indices(struct Common* common, NCZSlice* slices, size64_t* chunkindices);
 
@@ -123,10 +122,8 @@ NCZ_transferslice(NC_VAR_INFO_T* var, int reading,
     common.typesize = typesize;
     common.cache = zvar->cache;
 
-    if((stat = ncz_get_fill_value(common.file, common.var, &common.fillvalue))) goto done;
-
     /* We need to talk scalar into account */
-    common.rank = var->ndims + zvar->scalar;
+    common.rank = var->ndims;
     common.scalar = zvar->scalar;
     common.swap = (zfile->native_endianness == var->endianness ? 0 : 1);
 
@@ -227,7 +224,6 @@ NCZ_transfer(struct Common* common, NCZSlice* slices)
 	/* Read the chunk */
         switch ((stat = common->reader.read(common->reader.source, chunkindices, &chunkdata))) {
         case NC_EEMPTY: /* cache created the chunk */
-            if((stat = NCZ_fillchunk(chunkdata,common))) goto done;
 	    break;
         case NC_NOERR: break;
         default: goto done;
@@ -299,7 +295,6 @@ NCZ_transfer(struct Common* common, NCZSlice* slices)
         stat = common->reader.read(common->reader.source, chunkindices, &chunkdata);
 	switch (stat) {
         case NC_EEMPTY: /* cache created the chunk */
-	    if((stat = NCZ_fillchunk(chunkdata,common))) goto done;
 	    break;
         case NC_NOERR: break;
         default: goto done;
@@ -503,6 +498,7 @@ unsigned srcidx = srcoff / sizeof(unsigned); (void)srcidx;
 }
 #endif
 
+#if 0
 /* This function may not be necessary if code in zvar does it instead */
 static int
 NCZ_fillchunk(void* chunkdata, struct Common* common)
@@ -523,6 +519,7 @@ NCZ_fillchunk(void* chunkdata, struct Common* common)
 done:
     return stat;
 }
+#endif
 
 /* Break out this piece so we can use it for unit testing */
 int
@@ -681,7 +678,6 @@ NCZ_clearcommon(struct Common* common)
 {
     NCZ_clearsliceprojections(common->rank,common->allprojections);
     nullfree(common->allprojections);
-    nullfree(common->fillvalue);
 }
 
 /* Does the User want all of one and only chunk? */
@@ -730,7 +726,6 @@ NCZ_transferscalar(struct Common* common)
     chunkindices[0] = 0;
     switch ((stat = common->reader.read(common->reader.source, chunkindices, &chunkdata))) {
     case NC_EEMPTY: /* cache created the chunk */
-        if((stat = NCZ_fillchunk(chunkdata,common))) goto done;
 	break;
     case NC_NOERR: break;
     default: goto done;
