@@ -12,27 +12,28 @@
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
+
+#if defined(_WIN32) && !defined(__MINGW32__)
+#include "XGetopt.h"
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <string.h>
 #include "netcdf.h"
 #include "netcdf_filter.h"
+#include "netcdf_aux.h"
 #include "nciter.h"
 #include "utils.h"
 #include "chunkspec.h"
 #include "dimmap.h"
 #include "nccomps.h"
 #include "list.h"
+#include "ncpathmgr.h"
 
 #undef DEBUGFILTER
-
-#ifdef _MSC_VER
-#include "XGetopt.h"
-#define snprintf _snprintf
-int opterr;
-int optind;
-#endif
+#undef DEBUGCHUNK
 
 /* default bytes of memory we are willing to allocate for variable
  * values during copy */
@@ -62,19 +63,19 @@ typedef struct VarID {
 #define MAX_FILTER_SPECS 64
 #define MAX_FILTER_PARAMS 256
 
-struct FilterSpec {
+struct FilterOption {
     char* fqn; /* Of variable */
     int nofilter; /* 1=> do not apply any filters to this variable */
-    NC4_Filterspec pfs;
+    NC_H5_Filterspec pfs;
 };
 
-static List* filterspecs = NULL;
+static List* filteroptions = NULL;
 static int suppressfilters = 0; /* 1 => do not apply any output filters unless specified */
 
 /* Forward declaration, because copy_type, copy_vlen_type call each other */
 static int copy_type(int igrp, nc_type typeid, int ogrp);
-static void freespeclist(List* specs);
-static void freefilterlist(size_t,NC4_Filterspec**);
+static void freefilteroptlist(List* specs);
+static void freefilterlist(size_t nfilters, NC_H5_Filterspec** filters);
 
 #endif
 
@@ -152,7 +153,6 @@ static char** option_lvars = 0;		/* list of variable names specified with -v
 static bool_t option_varstruct = false;	  /* if -v set, copy structure for non-selected vars */
 static int option_compute_chunkcaches = 0; /* default, don't try still flaky estimate of
 					    * chunk cache for each variable */
-
 /* get group id in output corresponding to group igrp in input,
  * given parent group id (or root group id) parid in output. */
 static int
@@ -242,7 +242,7 @@ computeFQN(VarID vid, char** fqnp)
     *q++ = '\0'; /* guarantee */
     strcat(fqn,escname);
 done:
-    if(stat == NC_NOERR && fqnp != NULL) *fqnp = fqn;
+    if(stat == NC_NOERR && fqnp != NULL) {*fqnp = fqn; fqn = NULL;}
     return stat;
 }
 
@@ -290,7 +290,7 @@ parsefilterspec(const char* optarg0, List* speclist)
     int i;
     int isnone = 0;
     size_t nfilters = 0;
-    NC4_Filterspec** filters = NULL;
+    NC_H5_Filterspec** filters = NULL;
 
     if(optarg0 == NULL || strlen(optarg0) == 0 || speclist == NULL) return 0;
     optarg = strdup(optarg0);
@@ -305,61 +305,58 @@ parsefilterspec(const char* optarg0, List* speclist)
     }
     /* Parse the variable list */
     if((vlist = listnew()) == NULL) {stat = NC_ENOMEM; goto done;}
-    if((stat=parsevarlist(optarg,vlist))) goto done;        
+    if((stat=parsevarlist(optarg,vlist))) goto done;
 
     if(strcasecmp(remainder,"none") != 0) {
-	int format;
         /* Collect the id+parameters */
-        if((stat=NC_parsefilterlist(remainder,&format,&nfilters,(NC_Filterspec***)&filters))) goto done;
-	if(format != NC_FILTER_FORMAT_HDF5)
-	    {stat = NC_EFILTER; goto done;}
+        if((stat=ncaux_h5filterspec_parselist(remainder,NULL,&nfilters,&filters))) goto done;
     } else {
         isnone = 1;
         if(nfilters == 0) {
 	    /* Add a fake filter */
-	    NC4_Filterspec* nilspec = (NC4_Filterspec*)calloc(1,sizeof(NC4_Filterspec));
+	    NC_H5_Filterspec* nilspec = (NC_H5_Filterspec*)calloc(1,sizeof(NC_H5_Filterspec));
 	    if(nilspec == NULL) {stat = NC_ENOMEM; goto done;}
 	    nfilters = 1;
-	    filters = calloc(1,sizeof(NC4_Filterspec**));
+	    filters = calloc(1,sizeof(NC_H5_Filterspec**));
 	    if(filters == NULL) {free(nilspec); stat = NC_ENOMEM; goto done;}
 	    filters[0] = nilspec; nilspec = NULL;
 	}
     }
-    
+
     /* Construct a spec entry for each element in vlist */
     for(i=0;i<listlength(vlist);i++) {
 	int k;
 	size_t vlen;
-        struct FilterSpec* spec = NULL;
+        struct FilterOption* filtopt = NULL;
 	const char* var = listget(vlist,i);
 	if(var == NULL || strlen(var) == 0) continue;
 	vlen = strlen(var);
 	for(k=0;k<nfilters;k++) {
-	    NC4_Filterspec* nsf = filters[k];
-	    if((spec = calloc(1,sizeof(struct FilterSpec)))==NULL)
+	    NC_H5_Filterspec* nsf = filters[k];
+	    if((filtopt = calloc(1,sizeof(struct FilterOption)))==NULL)
 	       {stat = NC_ENOMEM; goto done;}
-	    spec->fqn = malloc(vlen+1+1); /* make room for nul and possible prefix '/' */
-	    if(spec->fqn == NULL) {stat = NC_ENOMEM; goto done;}
-            spec->fqn[0] = '\0'; /* for strlcat */
-	    if(strcmp(var,"*") != 0 && var[0] != '/') strlcat(spec->fqn,"/",vlen+2);
-	    strlcat(spec->fqn,var,vlen+2);
+	    filtopt->fqn = malloc(vlen+1+1); /* make room for nul and possible prefix '/' */
+	    if(filtopt->fqn == NULL) {stat = NC_ENOMEM; goto done;}
+            filtopt->fqn[0] = '\0'; /* for strlcat */
+	    if(strcmp(var,"*") != 0 && var[0] != '/') strlcat(filtopt->fqn,"/",vlen+2);
+	    strlcat(filtopt->fqn,var,vlen+2);
    	    if(isnone)
-	        spec->nofilter = 1;
+	        filtopt->nofilter = 1;
 	    else {
- 	        spec->pfs = *nsf;
+ 	        filtopt->pfs = *nsf;
 		if(nsf->nparams != 0) {
 	            /* Duplicate the params */
-	            spec->pfs.params = malloc(spec->pfs.nparams*sizeof(unsigned int));
-	            if(spec->pfs.params == NULL) {stat = NC_ENOMEM; goto done;}
-	            memcpy(spec->pfs.params,nsf->params,spec->pfs.nparams*sizeof(unsigned int));
+	            filtopt->pfs.params = calloc(filtopt->pfs.nparams,sizeof(unsigned int));
+	            if(filtopt->pfs.params == NULL) {stat = NC_ENOMEM; goto done;}
+		    memcpy(filtopt->pfs.params,nsf->params,sizeof(unsigned int)*filtopt->pfs.nparams);
 		} else
-		    spec->pfs.params = NULL;
+		    filtopt->pfs.params = NULL;
 	    }
-  	    listpush(speclist,spec);
-	    spec = NULL;
+  	    listpush(filteroptions,filtopt);
+	    filtopt = NULL;
 	}
     }
-    
+
 done:
     freefilterlist(nfilters,filters);
     if(vlist) listfreeall(vlist);
@@ -367,33 +364,50 @@ done:
     return stat;
 }
 
+/* Return 1 if variable has only active (ie not none) filters */
 static int
-hasfilterspecforvar(const char* ofqn)
+varfiltersactive(const char* ofqn)
 {
-  int i;	 
-  /* See which output filter specs are defined for this output variable */
-  for(i=0;i<listlength(filterspecs);i++) {
-      struct FilterSpec* spec = listget(filterspecs,i);
-      if(strcmp(spec->fqn,"*")==0) return 1;
-      if(strcmp(spec->fqn,ofqn)==0) return 1;
+  int i;
+  int hasnone = 0;
+  int hasactive = 0;
+  /* See which output filter options are defined for this output variable */
+  for(i=0;i<listlength(filteroptions);i++) {
+      struct FilterOption* opt = listget(filteroptions,i);
+      if(strcmp(opt->fqn,"*")==0 || strcmp(opt->fqn,ofqn)==0)
+	{if(opt->nofilter) hasnone = 1;} else {hasactive = 1;}
   }
-  return 0;
+  return (hasactive && !hasnone ? 1 : 0);
 }
 
-static List*
-filterspecsforvar(const char* ofqn)
+/* Return 1 if variable has "none" filters */
+static int
+varfilterssuppress(const char* ofqn)
 {
-  int i;	 
+  int i;
+  int hasnone = 0;
+  /* See which output filter options are defined for this output variable */
+  for(i=0;i<listlength(filteroptions);i++) {
+      struct FilterOption* opt = listget(filteroptions,i);
+      if(strcmp(opt->fqn,"*")==0 || strcmp(opt->fqn,ofqn)==0)
+	{if(opt->nofilter) hasnone = 1;}
+  }
+  return hasnone || suppressfilters;
+}
+
+/* Return list of active filters */
+static List*
+filteroptsforvar(const char* ofqn)
+{
+  int i;
   List* list = listnew();
-  /* See which output filter specs are defined for this output variable */
-  for(i=0;i<listlength(filterspecs);i++) {
-      struct FilterSpec* spec = listget(filterspecs,i);
-      if(strcmp(spec->fqn,"*")==0) {
-	  /* Add to the list */
-	  listpush(list,spec);
-      } else if(strcmp(spec->fqn,ofqn)==0) {
-	  /* Add to the list */
-	  listpush(list,spec);
+  /* See which output filter options are defined for this output variable;
+     both active and none. */
+  for(i=0;i<listlength(filteroptions);i++) {
+      struct FilterOption* opt = listget(filteroptions,i);
+      if(strcmp(opt->fqn,"*")==0 || strcmp(opt->fqn,ofqn)==0) {
+	if(!opt->nofilter) /* Add to the list */
+	  listpush(list,opt);
       }
   }
   return list;
@@ -766,12 +780,13 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
     List* ospecs = NULL;
     List* inspecs = NULL;
     List* actualspecs = NULL;
-    struct FilterSpec inspec;
-    struct FilterSpec* tmp = NULL;
+    struct FilterOption inspec;
+    struct FilterOption* tmp = NULL;
     char* ofqn = NULL;
     int inputdefined, outputdefined, unfiltered;
     int innc4 = (inkind == NC_FORMAT_NETCDF4 || inkind == NC_FORMAT_NETCDF4_CLASSIC);
     int outnc4 = (outkind == NC_FORMAT_NETCDF4 || outkind == NC_FORMAT_NETCDF4_CLASSIC);
+    int suppressvarfilters = 0;
 
     if(!outnc4)
 	goto done; /* Can only use filter when output is some netcdf4 variant */
@@ -784,40 +799,41 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
     ospecs = NULL;
     actualspecs = NULL;
 
+    if(varfilterssuppress(ofqn) || option_deflate_level == 0)
+        suppressvarfilters = 1;
+
     /* Is there one or more filters on the output variable */
     outputdefined = 0; /* default is no filter defined */
-    /* Only bother to look if output is netcdf-4 variant */
-    if(outnc4) {
-      /* See if any output filter spec is defined for this output variable */
-      ospecs = filterspecsforvar(ofqn);
-      if(listlength(ospecs) > 0)
+    /* See if any output filter spec is defined for this output variable */
+    ospecs = filteroptsforvar(ofqn);
+    if(listlength(ospecs) > 0 && !suppressfilters && !suppressvarfilters)
           outputdefined = 1;
-    }
 
     /* Is there already a filter on the input variable */
     inputdefined = 0; /* default is no filter defined */
     /* Only bother to look if input is netcdf-4 variant */
     if(innc4) {
       size_t nfilters;
-      unsigned int* ids = NULL;      
+      unsigned int* ids = NULL;
       int k;
-      if((stat = nc_inq_var_filterids(vid.grpid,vid.varid,&nfilters,NULL)))
+      if((stat = nc_inq_var_filter_ids(vid.grpid,vid.varid,&nfilters,NULL)))
 	goto done;
-      if(nfilters > 0) ids = malloc(nfilters*sizeof(unsigned int));
-      if((stat = nc_inq_var_filterids(vid.grpid,vid.varid,&nfilters,ids)))
+      if(nfilters > 0) ids = (unsigned int*)calloc(nfilters,sizeof(unsigned int));
+      if((stat = nc_inq_var_filter_ids(vid.grpid,vid.varid,&nfilters,ids)))
 	goto done;
       memset(&inspec,0,sizeof(inspec));
+
       for(k=0;k<nfilters;k++) {
 	  inspec.pfs.filterid = ids[k];
           stat=nc_inq_var_filter_info(vid.grpid,vid.varid,inspec.pfs.filterid,&inspec.pfs.nparams,NULL);
           if(stat && stat != NC_ENOFILTER)
 	    goto done; /* true error */
           if(inspec.pfs.nparams > 0) {
-            inspec.pfs.params = (unsigned int*)malloc(sizeof(unsigned int)*inspec.pfs.nparams);
+	    inspec.pfs.params = (unsigned int*)calloc(sizeof(char*),inspec.pfs.nparams);
             if((stat=nc_inq_var_filter_info(vid.grpid,vid.varid,inspec.pfs.filterid,NULL,inspec.pfs.params)))
 	       goto done;
 	  }
-          tmp = malloc(sizeof(struct FilterSpec));
+          tmp = malloc(sizeof(struct FilterOption));
           *tmp = inspec;
           memset(&inspec,0,sizeof(inspec)); /*reset*/
           listpush(inspecs,tmp);
@@ -826,7 +842,7 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
 	nullfree(ids);
     }
 
-    /* Rules for choosing output filter are as follows:
+    /* Rules for choosing output filter are as follows (Ugh!):
 
 	   global	output		input	  Actual Output
 	   suppress	filter(s)	filter(s) filter
@@ -844,27 +860,18 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
     unfiltered = 0;
     if(suppressfilters && !outputdefined) /* row 1 */
       unfiltered = 1;
-    else if(suppressfilters && outputdefined) { /* row 2 */
-	int k;
-        /* Walk the set of filters to apply to see if "none" was specified */
-        for(k=0;k<listlength(ospecs);k++) {
-	    struct FilterSpec* sp = (struct FilterSpec*)listget(actualspecs,k);
-	    if(sp->nofilter) {unfiltered = 1; break;}
-        }
-    } else if(suppressfilters && outputdefined) /* row 3 */
-      actualspecs = ospecs;
+    else if(suppressfilters || suppressvarfilters) /* row 2 */
+       unfiltered = 1;
+    else if(suppressfilters && outputdefined) /* row 3 */
+       actualspecs = ospecs;
     else if(!suppressfilters && !outputdefined && inputdefined) /* row 4 */
-      actualspecs = inspecs;
-    else if(!suppressfilters && outputdefined) { /* row 5 & 6*/
-	int k;
-        /* Walk the set of filters to apply to see if "none" was specified */
-        for(k=0;k<listlength(ospecs);k++) {
-	    struct FilterSpec* sp = (struct FilterSpec*)listget(ospecs,k);
-	    if(sp->nofilter) {unfiltered = 1; break;}
-        }
-	if(!unfiltered) actualspecs = ospecs;
-    } else if(!suppressfilters && !outputdefined && !inputdefined) /* row 7 */
-      unfiltered = 1;
+       actualspecs = inspecs;
+    else if(!suppressfilters && suppressvarfilters) /* row 5 */
+       unfiltered = 1;
+    else if(!suppressfilters && outputdefined) /* row 6*/
+       actualspecs = ospecs;
+    else if(!suppressfilters && !outputdefined && !inputdefined) /* row 7 */
+       unfiltered = 1;
     else if(!suppressfilters && outputdefined && inputdefined) /* row 8 */
       actualspecs = ospecs;
 
@@ -873,7 +880,7 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
 	/* add all the actual filters */
 	int k;
 	for(k=0;k<listlength(actualspecs);k++) {
-	    struct FilterSpec* actual = (struct FilterSpec*)listget(actualspecs,k);
+	    struct FilterOption* actual = (struct FilterOption*)listget(actualspecs,k);
 	    if((stat=nc_def_var_filter(ovid.grpid,ovid.varid,
 				   actual->pfs.filterid,
 				   actual->pfs.nparams,
@@ -884,7 +891,7 @@ copy_var_filter(int igrp, int varid, int ogrp, int o_varid, int inkind, int outk
 done:
     /* Cleanup */
     if(ofqn != NULL) free(ofqn);
-    freespeclist(inspecs); inspecs = NULL;
+    freefilteroptlist(inspecs); inspecs = NULL;
     listfree(ospecs); ospecs = NULL; /* Contents are also in filterspecs */
     /* Note we do not clean actualspec because it is a copy of in|out spec */
     return stat;
@@ -906,6 +913,8 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
     size_t ichunkp[NC_MAX_VAR_DIMS];
     size_t ochunkp[NC_MAX_VAR_DIMS];
     size_t dimlens[NC_MAX_VAR_DIMS];
+    size_t perdimchunklen[NC_MAX_VAR_DIMS]; /* the values of relevant -c dim/n specifications */
+    size_t dfaltchunkp[NC_MAX_VAR_DIMS]; /* default chunking for ovarid */
     int is_unlimited = 0;
 
     /* First, check the file kinds */
@@ -915,6 +924,8 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
     memset(ichunkp,0,sizeof(ichunkp));
     memset(ochunkp,0,sizeof(ochunkp));
     memset(dimlens,0,sizeof(dimlens));
+    memset(perdimchunklen,0,sizeof(perdimchunklen));
+    memset(dfaltchunkp,0,sizeof(dfaltchunkp));
 
     /* Get the chunking, if any, on the current input variable */
     if(innc4) {
@@ -922,7 +933,7 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
 	    /* pretend that this is same as a -c option */
     } else { /* !innc4 */
 	icontig = NC_CONTIGUOUS;
-	ichunkp[0] = 0;	    	
+	ichunkp[0] = 0;
     }
 
     /* If var specific chunking was specified for this output variable
@@ -950,10 +961,10 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
     ovid.grpid = ogrp;
     ovid.varid = o_varid;
     if((stat=computeFQN(ovid,&ofqn))) goto done;
-    if(option_deflate_level >= 0 || hasfilterspecforvar(ofqn))
+    if(!varfilterssuppress(ofqn) && (option_deflate_level > 0 || varfiltersactive(ofqn)))
 	ocontig = NC_CHUNKED;
 
-    /* See about dim-specific chunking; does not override -c spec*/
+    /* See about dim-specific chunking; does not override specific variable chunk spec*/
     {
 	int idim;
 	/* size of a chunk: product of dimension chunksizes and size of value */
@@ -962,8 +973,10 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
 	int dimids[NC_MAX_VAR_DIMS];
 
 	/* See if dim-specific chunking was suppressed */
-	if(dimchunkspec_omit()) /* use input chunksizes */
+	if(dimchunkspec_omit()) { /* no chunking at all on output, except as overridden by e.g. compression */
+	   ocontig = NC_CONTIGUOUS;
     	    goto next2;
+	}
 
 	/* Setup for possible output chunking */
 	typesize = val_size(igrp, i_varid);
@@ -973,8 +986,8 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
 	/* Prepare to iterate over the dimids of this input variable */
 	NC_CHECK(nc_inq_vardimid(igrp, i_varid, dimids));
 
-        /* Capture dimension lengts for all dimensions of variable;
-	   even if we decide to not chunk */
+        /* Capture dimension lengths for all dimensions of variable */
+	/* Also, capture per-dimension -c specs even if we decide to not chunk */
         for(idim = 0; idim < ndims; idim++) {
             int idimid = dimids[idim];
             int odimid = dimmap_odimid(idimid);
@@ -989,8 +1002,8 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
 	    }
 
 	    if(dimchunkspec_exists(idimid)) {
-                /* If the -c set a chunk size for this dimension, use it */
-                dimlens[idim] = dimchunkspec_size(idimid); /* Save it */
+                /* If the -c set a chunk size for this dimension, capture it */
+                perdimchunklen[idim] = dimchunkspec_size(idimid); /* Save it */
 		ocontig = NC_CHUNKED; /* force chunking */
 	    }
 
@@ -1001,13 +1014,35 @@ copy_chunking(int igrp, int i_varid, int ogrp, int o_varid, int ndims, int inkin
                     dimlens[idim] = mb4dimsize;
             }
 	}
-	/* compute the final ochunksizes: precedence is output, input, dimeln */
+
+        /* Get the current default chunking on the output variable */
+        /* Unfortunately, there is no way to get this info except by
+           forcing chunking */
+        if(ocontig == NC_CHUNKED) {
+	    /* this may fail if chunking is not possible, in which case ignore */
+            int ret = nc_def_var_chunking(ogrp, o_varid, NC_CHUNKED, dfaltchunkp);
+	    if(ret == NC_NOERR) {
+		int storage;
+	        NC_CHECK(nc_inq_var_chunking(ogrp, o_varid, &storage, dfaltchunkp));
+		if(storage != NC_CHUNKED) return NC_EINTERNAL;
+	    }
+        }
+
+	/* compute the final ochunksizes: precedence is output, per-dim-spec, input, defaults, dimlen */
         for(idim = 0; idim < ndims; idim++) {
-	    if(ochunkp[idim] == 0) {
+	    if(ochunkp[idim] == 0) { /* use -c dim/n if specified */
+	        if(perdimchunklen[idim] != 0)
+		    ochunkp[idim] = perdimchunklen[idim];
+	    }
+	    if(ochunkp[idim] == 0) { /* use input chunk size */
 	        if(ichunkp[idim] != 0)
 		    ochunkp[idim] = ichunkp[idim];
 	    }
-	    if(ochunkp[idim] == 0) {
+	    if(ochunkp[idim] == 0) { /* use chunk defaults */
+	        if(dfaltchunkp[idim] != 0)
+		    ochunkp[idim] = dfaltchunkp[idim];
+	    }
+	    if(ochunkp[idim] == 0) { /* last resort: use full dimension size */
 	        if(dimlens[idim] != 0)
 		    ochunkp[idim] = dimlens[idim];
 	    }
@@ -1054,7 +1089,7 @@ next2:
     }
 #endif /*DEBUGFILTER*/
 #endif /*USE_NETCDF4*/
- 
+
 done:
     if(ofqn) free(ofqn);
     return stat;
@@ -1070,6 +1105,9 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
     int outnc4 = (outkind == NC_FORMAT_NETCDF4 || outkind == NC_FORMAT_NETCDF4_CLASSIC);
     int deflated = 0; /* true iff deflation is applied */
     int ndims;
+    char* ofqn = NULL;
+    int nofilters = 0;
+    VarID ovid = {ogrp,o_varid};
 
     if(!outnc4)
 	return stat; /* Ignore non-netcdf4 files */
@@ -1081,7 +1119,10 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
 	}
     }
 
-    if(ndims > 0)
+    if((stat=computeFQN(ovid,&ofqn))) goto done;
+    nofilters = varfilterssuppress(ofqn);
+
+    if(ndims > 0 && !nofilters)
     { /* handle compression parameters, copying from input, overriding
        * with command-line options */
 	int shuffle_in=0, deflate_in=0, deflate_level_in=0;
@@ -1103,7 +1144,7 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
 	    deflate_out = 0;
 	    deflate_level_out = 0;
 	}
-	/* Apply output deflation */
+	/* Apply output deflation (unless suppressed) */
 	if(outnc4) {
 	    /* Note that if we invoke this function and even if shuffle and deflate flags are 0,
                then default chunking will be turned on; so do a special check for that. */
@@ -1112,8 +1153,7 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
 	    deflated = deflate_out;
 	}
     }
-
-    if(innc4 && outnc4 && ndims > 0)
+    if(!nofilters && innc4 && outnc4 && ndims > 0)
     {				/* handle checksum parameters */
 	int fletcher32 = 0;
 	NC_CHECK(nc_inq_var_fletcher32(igrp, varid, &fletcher32));
@@ -1121,7 +1161,6 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
 	    NC_CHECK(nc_def_var_fletcher32(ogrp, o_varid, fletcher32));
 	}
     }
-
     if(innc4 && outnc4)
     {				/* handle endianness */
 	int endianness = 0;
@@ -1131,11 +1170,12 @@ copy_var_specials(int igrp, int varid, int ogrp, int o_varid, int inkind, int ou
 	}
     }
 
-    if(!deflated && ndims > 0) {
+    if(!nofilters && !deflated && ndims > 0) {
         /* handle other general filters */
         NC_CHECK(copy_var_filter(igrp, varid, ogrp, o_varid, inkind, outkind));
     }
-
+done:
+    if(ofqn) free(ofqn);
     return stat;
 }
 
@@ -1463,6 +1503,29 @@ copy_vars(int igrp, int ogrp)
     return stat;
 }
 
+#if DEBUGCHUNK
+static void
+report(int rank, size_t* start, size_t* count, void* buf)
+{
+    int i;
+    size_t prod = 1;
+    for(i=0;i<rank;i++) prod *= count[i];
+    fprintf(stderr,"start=");
+    for(i=0;i<rank;i++)
+	fprintf(stderr,"%s%ld",(i==0?"(":" "),(long)start[i]);
+    fprintf(stderr,")");
+    fprintf(stderr," count=");
+    for(i=0;i<rank;i++)
+	fprintf(stderr,"%s%ld",(i==0?"(":" "),(long)count[i]);
+    fprintf(stderr,")");
+    fprintf(stderr," data=");
+    for(i=0;i<prod;i++)
+	fprintf(stderr,"%s%d",(i==0?"(":" "),((int*)buf)[i]);
+    fprintf(stderr,"\n");
+    fflush(stderr);
+}
+#endif
+
 /* Copy the schema in a group and all its subgroups, recursively, from
  * group igrp in input to parent group ogrp in destination.  Use
  * dimmap array to map input dimids to output dimids. */
@@ -1526,7 +1589,8 @@ inq_nvals(int igrp, int varid, long long *nvalsp) {
 /* Copy data from variable varid in group igrp to corresponding group
  * ogrp. */
 static int
-copy_var_data(int igrp, int varid, int ogrp) {
+copy_var_data(int igrp, int varid, int ogrp)
+{
     int stat = NC_NOERR;
     nc_type vartype;
     long long nvalues;		/* number of values for this variable */
@@ -1615,6 +1679,9 @@ copy_var_data(int igrp, int varid, int ogrp) {
      * subsequent calls. */
     while((ntoget = nc_next_iter(iterp, start, count)) > 0) {
 	NC_CHECK(nc_get_vara(igrp, varid, start, count, buf));
+#ifdef DEBUGCHUNK
+	report(iterp->rank,start,count,buf);
+#endif
 	NC_CHECK(nc_put_vara(ogrp, ovarid, start, count, buf));
 #ifdef USE_NETCDF4
 	/* we have to explicitly free values for strings and vlens */
@@ -1988,16 +2055,16 @@ copy(char* infile, char* outfile)
 
 	/* Check if any vars in -v don't exist */
     if(missing_vars(igrp, option_nlvars, option_lvars))
-	exit(EXIT_FAILURE);
+	goto fail;
 
     if(option_nlgrps > 0) {
 	if(inkind != NC_FORMAT_NETCDF4) {
 	    error("Group list (-g ...) only permitted for netCDF-4 file");
-	    exit(EXIT_FAILURE);
+	    goto fail;
 	}
 	/* Check if any grps in -g don't exist */
 	if(grp_matches(igrp, option_nlgrps, option_lgrps, option_grpids) == 0)
-	    exit(EXIT_FAILURE);
+	    goto fail;
     }
 
     if(option_write_diskless)
@@ -2077,6 +2144,9 @@ copy(char* infile, char* outfile)
     NC_CHECK(nc_close(igrp));
     NC_CHECK(nc_close(ogrp));
     return stat;
+fail:
+    nc_finalize();
+    exit(EXIT_FAILURE);
 }
 
 /*
@@ -2131,7 +2201,7 @@ usage(void)
   [-5]      CDF5 output (same as -k 'cdf5)\n\
   [-d n]    set output deflation compression level, default same as input (0=none 9=max)\n\
   [-s]      add shuffle option to deflation compression\n\
-  [-c chunkspec] specify chunking for dimensions, e.g. \"dim1/N1,dim2/N2,...\"\n\
+  [-c chunkspec] specify chunking for variable and dimensions, e.g. \"var:N1,N2,...\" or \"dim1/N1,dim2/N2,...\"\n\
   [-u]      convert unlimited dimensions to fixed-size dimensions in output copy\n\
   [-w]      write whole output file from diskless netCDF on close\n\
   [-v var1,...] include data for only listed variables, but definitions for all variables\n\
@@ -2168,7 +2238,6 @@ main(int argc, char**argv)
     chunkspecinit();
     option_chunkspecs = listnew();
 
-    opterr = 1;
     progname = argv[0];
 
     if (argc <= 1)
@@ -2176,6 +2245,7 @@ main(int argc, char**argv)
        usage();
     }
 
+    opterr = 1;
     while ((c = getopt(argc, argv, "k:3467d:sum:c:h:e:rwxg:G:v:V:F:L:M:")) != -1) {
 	switch(c) {
         case 'k': /* for specifying variant of netCDF format to be generated
@@ -2205,6 +2275,7 @@ main(int argc, char**argv)
 		if(kvalue->name == NULL) {
 		    error("invalid output format: %s", kind_name);
 		}
+		nullfree(kind_name);
 	    }
 	    break;
 	case '3':		/* output format is classic (netCDF-3) */
@@ -2310,9 +2381,9 @@ main(int argc, char**argv)
 	       || strcasecmp(optarg,"*,none")==0) {
 		suppressfilters = 1;
 	    } else {
-		if(filterspecs == NULL)
-		    filterspecs = listnew();
-		NC_CHECK(parsefilterspec(optarg,filterspecs));
+		if(filteroptions == NULL)
+		    filteroptions = listnew();
+		NC_CHECK(parsefilterspec(optarg,filteroptions));
 		/* Force output to be netcdf-4 */
 		option_kind = NC_FORMAT_NETCDF4;
 	    }
@@ -2341,9 +2412,9 @@ main(int argc, char**argv)
     if (argc != 2) {
 	error("one input file and one output file required");
     }
-    inputfile = argv[0];
-    outputfile = argv[1];
-
+    /* Canonicalize the input and output files names */
+    inputfile = NC_shellUnescape(argv[0]); /* Remove shell added escapes */
+    outputfile = NC_shellUnescape(argv[1]);
     if(strcmp(inputfile, outputfile) == 0) {
 	error("output would overwrite input");
     }
@@ -2369,40 +2440,43 @@ main(int argc, char**argv)
     if(copy(inputfile, outputfile) != NC_NOERR)
         exitcode = EXIT_FAILURE;
 
+    nullfree(inputfile);
+    nullfree(outputfile);
+
 #ifdef USE_NETCDF4
     /* Clean up */
-    freespeclist(filterspecs);
-    filterspecs = NULL;
+    freefilteroptlist(filteroptions);
+    filteroptions = NULL;
 #endif /*USE_NETCDF4*/
 
+    nc_finalize();
+
     exit(exitcode);
 }
 
 #ifdef USE_NETCDF4
 static void
-freespeclist(List* specs)
+freefilteroptlist(List* specs)
 {
     int i;
     for(i=0;i<listlength(specs);i++) {
-        struct FilterSpec* spec = (struct FilterSpec*)listget(specs,i);
+        struct FilterOption* spec = (struct FilterOption*)listget(specs,i);
         if(spec->fqn) free(spec->fqn);
-        if(spec->pfs.params) free(spec->pfs.params);
+        nullfree(spec->pfs.params);
         free(spec);
     }
     listfree(specs);
 }
 
 static void
-freefilterlist(size_t nfilters, NC4_Filterspec** filters)
+freefilterlist(size_t nfilters, NC_H5_Filterspec** filters)
 {
     int i;
-    if(filters == NULL) return;
-    for(i=0;i<nfilters;i++) {
-	NC4_Filterspec* pfs = filters[i];
-	if(pfs->params) free(pfs->params);
-	free(pfs);
+    if(filters != NULL) {
+        for(i=0;i<nfilters;i++)
+            ncaux_h5filterspec_free(filters[i]);
+        nullfree(filters);
     }
-    free(filters);
 }
-#endif
 
+#endif
